#include <math.h>
#include <GL/glut.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "Globals.h"
#include "Keys.h"
#include "SpaceSubwindow.h"
#include "GraphSubwindow.h"
#include "TextureUtils.h"
#include "Textures.h"

#include "Pass.h"
#include "Screen.h"

//#define DATA_FILE_NAME "2007-09-18_1200-CP3-CDH-parsed.csv"
//#define DATA_FILE_NAME "simpleData.csv"
//#define DATA_FILE_NAME "CP3-CDH-parsed.csv"
#define DATA_FILE_NAME "CP3-ADCS-parsed.csv"
//#define DATA_FILE_NAME "CP3-ADCS-parsed-cut.csv"
//#define DATA_FILE_NAME "CP3-ADCS-parsed-holes.csv"


// Window Parameters
#define WINDOW_BORDER 10
#define SUBWINDOW_WIDTH 1200

#define WINDOW_WIDTH  SUBWINDOW_WIDTH + (2*WINDOW_BORDER)
#define WINDOW_HEIGHT 800 + (3*WINDOW_BORDER)

#define SUBWINDOW_HEIGHT (WINDOW_HEIGHT - (3*WINDOW_BORDER)) / 2

int mainWindow, spaceWindow, graphWindow;


//float *BG_COLOR = LIGHT_GREY;
float BG_COLOR[] = {0.75f,0.75f,0.75f};

// GRAPH PARAMS
Pass *pass;


/* FUNCTION PROTOTYPES */
void reshapeCallback(int w, int h);
void mouseCallback(int button, int state, int x, int y);
void keyCallback(unsigned char key, int x, int y);
void keyCallbackSpecial(int key, int x, int y);
void changeSize(int w1, int h1);


void display (void) {
    glClearColor(1.0f, 0.0f, 1.0f,1.0f);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
}

void displayAll() {
    glutSetWindow(mainWindow);
    display();
    
    glutSetWindow(spaceWindow);
    spaceDisplay();
    
    glutSetWindow(graphWindow);
    graphDisplay();
}


void reshapeCallback(int w, int h) {

}
#define KEY_ENTER       13
#define KEY_ESC         27
#define KEY_BACKSLASH   92

void keyCallback(unsigned char key, int x, int y) {
    // Send to different functions
    
    //printf("%d\n", key);
    
    if (key == KEY_ESC) {
        exit(0);
    }
    
    if (key == KEY_TOGGLE_ORBIT_LOWER || key == KEY_TOGGLE_ORBIT_UPPER ||
        key == KEY_TOGGLE_EARTH_LOWER || key == KEY_TOGGLE_EARTH_UPPER ||
        key == KEY_TOGGLE_STARS_LOWER || key == KEY_TOGGLE_STARS_UPPER /*||
        key == KEY_TOGGLE_ORBIT_CAM*/
        ) {
       spaceKeyCallback(key,x,y);
    }
    
    if (key == KEY_MOVE_X_LEFT || key == KEY_MOVE_X_RIGHT || key == KEY_MOVE_Z_FORWARD ||
        key == KEY_MOVE_Z_BACK || key == KEY_MOVE_Y_UP || key == KEY_MOVE_Y_DOWN) {
        graphKeyCallback(key,x,y);
    }
    else if (key == KEY_TOGGLE_GRAPH_LOWER || key == KEY_TOGGLE_GRAPH_UPPER ||
        key == KEY_ENTER || key == KEY_SHOWN_LOWER || key == KEY_SHOWN_UPPER || key == 't' || key == 'T' ||
        key == KEY_CLEAR_DISPLAYED_LOWER || key == KEY_CLEAR_DISPLAYED_UPPER ||
        key == KEY_ALL_DISPLAYED_LOWER || key == KEY_ALL_DISPLAYED_UPPER ||
        key == KEY_REORDER_LOWER || key == KEY_REORDER_UPPER ||
        key == KEY_SELECT_COLOR_GLYPH || key == KEY_SELECT_SIZE_GLYPH ||
        key == KEY_GLYPH_SEL_LOWER || key == KEY_GLYPH_SEL_UPPER ||
        key == KEY_TILT_DOWN || key == KEY_TILT_UP ||
        key == KEY_TOGGLE_GRAPH_CAM
        ) {
        graphKeyCallback(key,x,y);
    }
    else if (key == '6'){
        offsetForText += 5;
    }
     else if (key == '7'){
        offsetForText -= 5;
    }
    

    displayAll();
}

void keyCallbackSpecial(int key, int x, int y) {
    if(key == GLUT_KEY_UP || key == GLUT_KEY_DOWN) {
        graphKeyCallbackSpecial(key, x, y);
    }
    else if(key == GLUT_KEY_LEFT) {
        // Decrement global index
        curSelectedIndex--;
        
        if(curSelectedIndex < 0){
            curSelectedIndex = pass->numDataPoints-1;
        }
    }
    else if(key == GLUT_KEY_RIGHT) {
        // Increament global index
        curSelectedIndex++;
        
        if(curSelectedIndex >= pass->numDataPoints){
            curSelectedIndex = 0;
        }
    }

    //spaceDisplay();
    //graphDisplay();
    //glutPostRedisplay();
    displayAll();
}

void changeSize(int w1, int h1) {

}

void mouseCallback(int button, int state, int x, int y) {
    spaceMouseCallback(button, state, x, y);
    
    displayAll();
}


void setupTextures() {
    initTextures();
    
    loadTexture("textures/earth.bmp", TEXTURE_EARTH);
    loadTexture("textures/solarPanelTex.bmp", TEXTURE_SOLAR_PANEL);
    loadTexture("textures/sunTex.bmp", TEXTURE_SUN);
    
    
}


void update(int value) {         
    glutPostRedisplay();         
    glutTimerFunc(100, update, 0);        
} 

//general intialization for opengl for depth and texture mapping
void init() {
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_TEXTURE_2D);
}

int main(int argc, char* argv[]) {
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
    glutInitWindowSize(WINDOW_WIDTH, WINDOW_HEIGHT);
    glutInitWindowPosition(5, 5);
    /*int tempMainWindow = 0;
    tempMainWindow = glutCreateWindow("Satellite Data Viewer");
    
    mainWindow = glutCreateSubWindow(tempMainWindow, 
        0,0, WINDOW_WIDTH, WINDOW_HEIGHT);*/
    
    //printf("%s\n", gluGetString(GLU_VERSION));
    
    mainWindow = glutCreateWindow("Satellite Data Viewer");
    
    glutFullScreen();

    glutReshapeFunc(reshapeCallback);
    glutKeyboardFunc(keyCallback);
    glutMouseFunc(mouseCallback);
    glutReshapeFunc(changeSize);
    glutDisplayFunc(display);
    glutSpecialFunc(keyCallbackSpecial);

    
        
    // Graph Window
    graphWindow = glutCreateSubWindow(mainWindow, 
        WINDOW_BORDER,(WINDOW_BORDER*2)+SUBWINDOW_HEIGHT,
        SUBWINDOW_WIDTH, SUBWINDOW_HEIGHT);
        
    
    glutDisplayFunc(graphDisplay);
    glutKeyboardFunc(keyCallback);
    glutReshapeFunc(graphReshapeCallback);
    glutMouseFunc(mouseCallback);
    glutReshapeFunc(graphChangeSize);
    glutSpecialFunc(keyCallbackSpecial);
    glEnable(GL_DEPTH_TEST);
    
    
    // Earth Window
    spaceWindow = glutCreateSubWindow(mainWindow, 
        WINDOW_BORDER,WINDOW_BORDER, SUBWINDOW_WIDTH, SUBWINDOW_HEIGHT);
    
    glutDisplayFunc(spaceDisplay);
    glutKeyboardFunc(keyCallback);
    glutReshapeFunc(spaceReshapeCallback);
    glutMouseFunc(mouseCallback);
    glutReshapeFunc(spaceChangeSize);
    glutSpecialFunc(keyCallbackSpecial);
    glEnable(GL_DEPTH_TEST);

    
    // Load in pass data
    pass = createPass(DATA_FILE_NAME);
    //printPass(pass);
    printf("Pass Info: numDataFields=%d , numDataPoints=%d\n", pass->numFields, pass->fields[0]->numData);
    
    graphSubwindowInit(pass);
    spaceSubwindowInit(pass);

    init();
    setupTextures();
    
    //glutTimerFunc(100, update, 0); 

    glutMainLoop();
    return 0;
}
