#include "Graph.h"

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <GL/glut.h>

#include "Arrow.h"
#include "DataField.h"
#include "DebugLog.h"
#include "Globals.h"
#include "OpenGLFunctions.h"
#include "Pass.h"
#include "PassIter.h"
#include "SatData.h"
#include "SatColors.h"
#include "Screen.h"

#define GRAPH_X_START -75.0
#define GRAPH_Y_START 0.0
#define GRAPH_Z_START 0.0

#define LABEL_X_START 10
#define LABEL_Y_START 20
#define LABEL_HEIGHT_SPACE 15

#define GRAPH_TYPE_X_START 300
#define GRAPH_TYPE_Y_START 20

// Variable for the pass iter
PassIter *passIter = NULL;

Screen *graphScreen;

int GRAPH_WIDTH = 1;

#define NUM_GRAPH_COLORS 6
float graphColors[NUM_GRAPH_COLORS][3] = 
{{1.0f,0.0,0.0}, {0.0,1.0f,0.0}, {0.0,0.0,1.0f}, {1.0f,0.0,1.0f},
{0.0,1.0f,1.0f}, {1.0f,1.0f,0.0}};
	
static int normalFont = (int) GLUT_BITMAP_8_BY_13;
static int subheaderFont = (int) GLUT_BITMAP_HELVETICA_18;
int bitmapHeight=13;

int selectedXCoord = 0;

// Private Prototypes
void drawDataFieldBar(DataField *dataField, float xStart, float yStart, float zStart, float graphDepth, float intervalSize, float barWidth, float* color);
//void drawDataFieldSmooth(DataField *dataField, float xStart, float yStart, float zStart, float graphDepth, float intervalSize, float* color);
void drawDataFieldConnected(DataField *dataField, float xStart, float yStart, float zStart, float graphDepth, float intervalSize, float* color);
void drawDataFieldLine(DataField *dataField, float xStart, float yStart, float zStart, float graphDepth, float intervalSize, float* color);
void drawTime(Pass *pass, float xStart, float yStart, float zStart, float intervalSize, float* color);
void drawLabels(Graph* graph);

void writeGraphType(Graph *g);

Graph *createGraph(Pass *pass, Screen *graphScreenToSet){
	Graph *toReturn = (Graph*) malloc(sizeof(Graph));
	
    graphScreen = graphScreenToSet;
    
	toReturn->pass = pass;
	toReturn->drawType = DRAW_TYPE_BAR;
	
	return toReturn;
}

void drawGraph(Graph* graph){
	int i = 0;
	int numShown = 0;

    printDebug("Starting drawGraph()");
    
    if(passIter == NULL){
        passIter = createPassIter(graph->pass);
    }
    
    for(i = 0; i < graph->pass->numFields; i++){
        if(graph->pass->fields[i]->display){
			if(graph->drawType == DRAW_TYPE_BAR) {
				drawDataFieldBar(graph->pass->fields[i], GRAPH_X_START, GRAPH_Y_START, GRAPH_Z_START-5*(numShown), GRAPH_WIDTH, 1, 0.5, graphColors[numShown%NUM_GRAPH_COLORS]);
			}
			else if(graph->drawType == DRAW_TYPE_CONNECTED) {
				drawDataFieldConnected(graph->pass->fields[i], GRAPH_X_START, GRAPH_Y_START, GRAPH_Z_START-5*(numShown), GRAPH_WIDTH, 1, graphColors[numShown%NUM_GRAPH_COLORS]);
			}
			else if(graph->drawType == DRAW_TYPE_LINE) {
				drawDataFieldLine(graph->pass->fields[i], GRAPH_X_START, GRAPH_Y_START, GRAPH_Z_START-5*(numShown), GRAPH_Z_START+GRAPH_WIDTH, 1, graphColors[numShown%NUM_GRAPH_COLORS]);
			}
			/*else if(graph->drawType == DRAW_TYPE_SMOOTH) {
				drawDataFieldSmooth(graph->pass->fields[i], 0, 0, 5*(numShown), GRAPH_WIDTH, 1, graphColors[numShown%NUM_GRAPH_COLORS]);
			}*/
			else {
				fprintf(stderr, "ERROR: Unsupported graph draw type: %d\n", graph->drawType);
			}
			
			numShown++;
		}
	}
	
    drawLabels(graph);
    
	// Draw time below
	drawTime(graph->pass,0,0,-100+offsetForText,GRAPH_WIDTH,COLOR_WHITE);

	writeGraphType(graph);
    
    // Draw the cur selected
    drawArrow(selectedXCoord, -2, 5, COLOR_WHITE);
    
    lastXDisForCur = selectedXCoord;
    
    printDebug("Ending drawGraph()");
}

/**
 * This function draws the times on the graph at a given interval
 */
void drawTime(Pass *pass, float xStart, float yStart, float zStart, float intervalSize, float* color){
	char tempStr[100];
    printDateTimeToArray(pass->fields[0]->data[curSelectedIndex]->dateTime, tempStr);
    
	glColor3fv(COLOR_WHITE);
    
    glDisable(GL_LIGHTING);
	
	setOrthographicProjection3D(graphScreen);
	glPushMatrix();
	glLoadIdentity(); 
	
	//printf("Drawing The Times %f, %f, %f\n", xStart, yStart, zStart);
	
	renderBitmapString3D(xStart, yStart, zStart,(void *)normalFont, tempStr);

	glPopMatrix();
	resetPerspectiveProjection3D();
    
    glEnable(GL_LIGHTING);
}

/**
 * This function draws the graph labels on the side of the graph for each axis.
 */
void drawLabels(Graph *graph) {
    int numShown = 0, i= 0;
	
	glDisable(GL_LIGHTING);
    
	setOrthographicProjection(graphScreen);
	glPushMatrix();
	glLoadIdentity();

    glColor3fv(COLOR_WHITE);
    renderBitmapString(LABEL_X_START,LABEL_Y_START,(void *)subheaderFont,"Fields Displayed");
    
    for(i = 0; i < graph->pass->numFields; i++){
        if(graph->pass->fields[i]->display){
            glColor3fv(graphColors[numShown%NUM_GRAPH_COLORS]);
            renderBitmapString(LABEL_X_START,LABEL_Y_START+20+(LABEL_HEIGHT_SPACE*numShown),(void *)normalFont,graph->pass->fields[i]->name);
            
            numShown++;
        }
    }


	glPopMatrix();
	resetPerspectiveProjection();
    
    glEnable(GL_LIGHTING);
}

void writeGraphType(Graph *g) {
	glColor3fv(COLOR_WHITE);
    
    glDisable(GL_LIGHTING);
  
	setOrthographicProjection(graphScreen);
	glPushMatrix();
	glLoadIdentity(); 
    
    renderBitmapString(GRAPH_TYPE_X_START,GRAPH_TYPE_Y_START,(void *)subheaderFont, "Graph Type:");
    
	if(g->drawType == DRAW_TYPE_BAR) {
		renderBitmapString(GRAPH_TYPE_X_START+110,GRAPH_TYPE_Y_START,(void *)normalFont, "Bar");
	}
	else if(g->drawType == DRAW_TYPE_CONNECTED) {
		renderBitmapString(GRAPH_TYPE_X_START+110,GRAPH_TYPE_Y_START,(void *)normalFont, "Connected");
	}
	else if(g->drawType == DRAW_TYPE_LINE) {
		renderBitmapString(GRAPH_TYPE_X_START+110,GRAPH_TYPE_Y_START,(void *)normalFont, "Line");
	}

	glPopMatrix();
	resetPerspectiveProjection();
    
    glEnable(GL_LIGHTING);
}


void drawDataFieldBar(DataField *dataField, float xStart, float yStart, float zStart, float graphDepth, float intervalSize, float barWidth, float* color) {
	int i = 0;
	DataValue **data = dataField->data;
    int curDataIndex = 0;
	int advResult = 0;
    
    printDebug("Starting drawDataFieldBar()");
    
    resetPassIter(passIter);
	
	glColor3fv(color);
	
	// Draw Top
	glBegin(GL_QUADS);
	
    for(i = 0; advResult >= 0; i++, advResult = advancePassIter(passIter)) {
        
        curDataIndex = samplePassIter(passIter);
        
        if(curDataIndex < 0) {
            //printf("No current data\n");
            continue;
        }

		// Draw Front
		glNormal3d(0, 0, 1);
		glVertex3f(xStart + ((float) i)*intervalSize, yStart, zStart);
		glVertex3f(xStart + ((float) i)*intervalSize + barWidth, yStart, zStart);
		glVertex3f(xStart + ((float) i)*intervalSize + barWidth, yStart+data[curDataIndex]->value, zStart);
		glVertex3f(xStart + ((float) i)*intervalSize, yStart+data[curDataIndex]->value, zStart);
        
		// Draw Left
		glNormal3d(-1, 0, 0);
		glVertex3f(xStart + ((float) i)*intervalSize, yStart, zStart);
		glVertex3f(xStart + ((float) i)*intervalSize, yStart+yStart+data[curDataIndex]->value, zStart);
		glVertex3f(xStart + ((float) i)*intervalSize, yStart+yStart+data[curDataIndex]->value, zStart+graphDepth);
		glVertex3f(xStart + ((float) i)*intervalSize, yStart, zStart+graphDepth);
		
		// Draw Back
		glNormal3d(0, 0, -1);
		glVertex3f(xStart + ((float) i)*intervalSize, yStart, zStart+graphDepth);
		glVertex3f(xStart + ((float) i)*intervalSize, yStart+data[curDataIndex]->value, zStart+graphDepth);
		glVertex3f(xStart + ((float) i)*intervalSize + barWidth, yStart+data[curDataIndex]->value, zStart+graphDepth);
		glVertex3f(xStart + ((float) i)*intervalSize + barWidth, yStart, zStart+graphDepth);
		
		// Draw Right
		glNormal3d(1, 0, 0);
		glVertex3f(xStart + ((float) i)*intervalSize+barWidth, yStart, zStart);
		glVertex3f(xStart + ((float) i)*intervalSize+barWidth, yStart, zStart+graphDepth);
		glVertex3f(xStart + ((float) i)*intervalSize+barWidth, yStart+data[curDataIndex]->value, zStart+graphDepth);
		glVertex3f(xStart + ((float) i)*intervalSize+barWidth, yStart+data[curDataIndex]->value, zStart);
				
		// Draw Top
		glNormal3d(0, 1, 0);
		glVertex3f(xStart + ((float) i)*intervalSize, yStart+data[curDataIndex]->value, zStart);
		glVertex3f(xStart + ((float) i)*intervalSize+barWidth, yStart+data[curDataIndex]->value, zStart);
		glVertex3f(xStart + ((float) i)*intervalSize+barWidth, yStart+data[curDataIndex]->value, zStart+graphDepth);
		glVertex3f(xStart + ((float) i)*intervalSize, yStart+data[curDataIndex]->value, zStart+graphDepth);
        
        if(curDataIndex == curSelectedIndex){
            selectedXCoord = xStart + ((float) i)*intervalSize;
        }
	}

	glEnd();

    //exit(1);    
    printDebug("Ending drawDataFieldBar()");
}


void drawDataFieldLine(DataField *dataField, float xStart, float yStart, float zStart, float graphDepth, float intervalSize, float* color) {
    // **@** need to set xselectedcoord
	int i = 0;
    int curDataIndex = 0;
	DataValue **data = dataField->data;
	int numDataPoints = dataField->numData;
	int stripOn = 1;
    int advResult = 0;
	
	glColor3fv(color);
	
    resetPassIter(passIter);
    
    // Draw Top
    glBegin(GL_QUAD_STRIP);
    
    // Left edge
    glVertex3f(xStart, yStart, zStart+graphDepth);
    glVertex3f(xStart, yStart, zStart);
    
    for(i = 0; advResult >= 0; i++, advResult = advancePassIter(passIter)) {
        
        curDataIndex = samplePassIter(passIter);
        
        if(curDataIndex < 0) {
            stripOn = 0;
            glEnd();
            //printf("No current data\n");
            continue;
        }
        
        if(stripOn == 0){
            stripOn = 1;
            glBegin(GL_QUAD_STRIP);
        }
        
        glVertex3f(xStart + ((float) i)*intervalSize, yStart+data[curDataIndex]->value, zStart+graphDepth);
        glVertex3f(xStart + ((float) i)*intervalSize, yStart+data[curDataIndex]->value, zStart);
        
        if(curDataIndex == curSelectedIndex){
            selectedXCoord = xStart + ((float) i)*intervalSize;
        }
    }

    
    
	// Draw Topdfgsfdgsdfgsfdg
    
    // Right edge
	glVertex3f(xStart + (numDataPoints-1)*intervalSize, yStart, zStart+graphDepth);
	glVertex3f(xStart + (numDataPoints-1)*intervalSize, yStart, zStart);
	
	glEnd();
}

void drawDataFieldConnected(DataField *dataField, float xStart, float yStart, float zStart, float graphDepth, float intervalSize, float* color) {
	int i = 0;
    int curDataIndex = 0;
	DataValue **data = dataField->data;
	int numDataPoints = dataField->numData;
    int stripOn = 1;
    int advResult = 0;
	
	glColor3fv(color);
	
    
    resetPassIter(passIter);
    
	// Draw Top
	glBegin(GL_QUAD_STRIP);
	
	glVertex3f(xStart, yStart, zStart+graphDepth);
	glVertex3f(xStart, yStart, zStart);
	
    stripOn = 1;
    advResult = 0;
    for(i = 0; advResult >= 0; i++, advResult = advancePassIter(passIter)) {
        
        curDataIndex = samplePassIter(passIter);
        
        if(curDataIndex < 0) {
            stripOn = 0;
            glEnd();
            //printf("No current data\n");
            continue;
        }
        
        if(stripOn == 0){
            stripOn = 1;
            glBegin(GL_QUAD_STRIP);
        }
        
		glVertex3f(xStart + ((float) i)*intervalSize, yStart+data[curDataIndex]->value, zStart+graphDepth);
		glVertex3f(xStart + ((float) i)*intervalSize, yStart+data[curDataIndex]->value, zStart);
        
        if(curDataIndex == curSelectedIndex){
            selectedXCoord = xStart + ((float) i)*intervalSize;
        }
	}
	
	glVertex3f(xStart + (i-1)*intervalSize, yStart, zStart+graphDepth);
	glVertex3f(xStart + (i-1)*intervalSize, yStart, zStart);
	
	glEnd();
	
    
    
    
    
    
    

    resetPassIter(passIter);
    

	// Draw Front Side
	glBegin(GL_QUAD_STRIP);
	
	glNormal3f(0.0,0.0,-1.0);
    
	stripOn = 1;
    advResult = 0;
    for(i = 0; advResult >= 0; i++, advResult = advancePassIter(passIter)) {
        
        curDataIndex = samplePassIter(passIter);
        
        if(curDataIndex < 0) {
            stripOn = 0;
            glEnd();
            continue;
        }
        
        if(stripOn == 0){
            stripOn = 1;
            glBegin(GL_QUAD_STRIP);
        }
        
		glVertex3f(xStart + ((float) i)*intervalSize, yStart, zStart);
		glVertex3f(xStart + ((float) i)*intervalSize, yStart+data[curDataIndex]->value, zStart);
	}
	
	glEnd();
	
    
    
    
    
    
    
    
    resetPassIter(passIter);
    

    // Draw Front Side
    glBegin(GL_QUAD_STRIP);
    
    glNormal3f(0.0,0.0,1.0);
    
    stripOn = 1;
    advResult = 0;
    for(i = 0; advResult >= 0; i++, advResult = advancePassIter(passIter)) {
        
        curDataIndex = samplePassIter(passIter);
        
        if(curDataIndex < 0) {
            stripOn = 0;
            glEnd();
            continue;
        }
        
        if(stripOn == 0){
            stripOn = 1;
            glBegin(GL_QUAD_STRIP);
        }
        
        glVertex3f(xStart + ((float) i)*intervalSize, yStart, zStart+graphDepth);
        glVertex3f(xStart + ((float) i)*intervalSize, yStart+data[curDataIndex]->value, zStart+graphDepth);
    }
    
    glEnd();
}
