#include "GraphSubwindow.h"

#include <GL/glut.h>
#include <stdio.h>

#include "Camera.h"
#include "Graph.h"
#include "Globals.h"
#include "Keys.h"
#include "MenuGlyph.h"
#include "MenuReorder.h"
#include "MenuShown.h"
#include "Pass.h"
#include "OpenGLFunctions.h"
#include "Screen.h"

Graph *graph;
Pass *pass;
MenuGlyph *menuGlyph;
MenuReorder *menuReorder;
MenuShown *menuShown;
Screen *graphScreen;

static CameraMode cameraMode = CAMERA_MODE_GLOBAL;

static int normalFont = (int) GLUT_BITMAP_8_BY_13;
static int subheaderFont = (int) GLUT_BITMAP_HELVETICA_18;

// State variables
//GraphSubwindowMode graphSubwindowMode = SET_VISIBLE_FIELDS;
GraphSubwindowMode graphSubwindowMode = DRAW_GRAPH;


//Prototypes
void graphSetUpView();
void graphSetUpModelTransform();
void graphSetUpLight();
void resetGraphCameraCoordinates();

void drawCurPassDateTimesOnGraphScreen();


// View variables
static GLfloat modelTrans[] = {-42.0, -5.0, -122.0};
//static GLfloat modelTrans[] = {-0.0, -0.0, -0.0};

static GLfloat theta[] = {20.0, 0.0, 0.0};
static float thetaIncr = 5.0;


void resetGraphCameraCoordinates() {    
    
    if(graph->drawType == DRAW_TYPE_LINE){
        // Always tracking
            modelTrans[0] = -lastXDisForCur;
            modelTrans[1] = 4.0;
            modelTrans[2] = -30.0;
            
            theta[0] = 0.0;
            theta[1] = 0;
            theta[2] = 0;
        
    } else {
        if(cameraMode == CAMERA_MODE_GLOBAL){
            // Global
            modelTrans[0] = -42.0;
            modelTrans[1] = -5.0;
            modelTrans[2] = -122.0;
            
            theta[0] = 20.0;
            theta[1] = 0;
            theta[2] = 0;
        } else {
            // Tracking

            modelTrans[0] = -lastXDisForCur;
            modelTrans[1] = -5.0;
            modelTrans[2] = -50.0;
            
            theta[0] = 25.0;
            theta[1] = 0;
            theta[2] = 0;
        }
    }
    
    
}

// animation transform variables
// static GLfloat translate[3] = {0.0, 0.0, 0.0};

void graphSetUpView() {
    // this code initializes the viewing transform
    glLoadIdentity();
    // moves viewer along coordinate axes
    gluLookAt(0.0, 0.0, 5.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
    // move the view back some relative to viewer[] position
    glTranslatef(0.0f,0.0f, -8.0f);
    // rotates view
    glRotatef(0, 1.0, 0.0, 0.0);
    glRotatef(0, 0.0, 1.0, 0.0);
    glRotatef(0, 0.0, 0.0, 1.0);

    return;
}

/*  Function name: setUpModelTransform()
    Description:   Sets up the initial model transform.
*/
void graphSetUpModelTransform() {
    
        // moves model along coordinate axes
        if(cameraMode == CAMERA_MODE_TRACKING || graph->drawType == DRAW_TYPE_LINE) {
            resetGraphCameraCoordinates();
        }
        
        glTranslatef(modelTrans[0], modelTrans[1], modelTrans[2]);
    
        // rotates model
        glRotatef(theta[0], 1.0, 0.0, 0.0);
        glRotatef(theta[1], 0.0, 1.0, 0.0);
        glRotatef(theta[2], 0.0, 0.0, 1.0);
    
}

/* Function name: setUpLight()
   Description:   Sets up the light source.
*/
void graphSetUpLight() {
    // set up the light sources for the scene
    // a directional light source from directly behind
    /* GLfloat lightDir[] = {0.0, 0.5, 1.0, 0.0};
    GLfloat diffuseComp[] = {1.0, 1.0, 1.0, 1.0};

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);

    glLightfv(GL_LIGHT0, GL_POSITION, lightDir);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseComp);*/

	GLfloat ambientLight[] = {0.1f, 0.1f, 0.1f, 1.0f};
    GLfloat diffuseLight[] = {0.5f, 0.5f, 0.5f, 1.0f};
   
    GLfloat lightDir[] = {0.0, 0.5, 1.0, 0.0};
    GLfloat diffuseComp[] = {1.0, 1.0, 1.0, 1.0};

    glEnable(GL_LIGHTING);


    glEnable(GL_LIGHTING);
    
    glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
    glEnable ( GL_COLOR_MATERIAL ) ;

    glLightfv(GL_LIGHT0, GL_AMBIENT, ambientLight);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);
    glEnable(GL_LIGHT0);

    // Old  


    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT1);

    glLightfv(GL_LIGHT1, GL_POSITION, lightDir);
    glLightfv(GL_LIGHT1, GL_DIFFUSE, diffuseComp);

    return;
}


void graphDisplay (void) {
    
    // this code executes whenever the window is redrawn (when opened,
    // moved, resized, etc.
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    // set the viewing transform
    graphSetUpView();

    // set up light source
    graphSetUpLight();

    // start drawing objects
    graphSetUpModelTransform();
    
    if (graphSubwindowMode == SET_VISIBLE_FIELDS) {
        drawMenuShown(menuShown);
    } else if (graphSubwindowMode == REORDER_FIELDS) {
        drawMenuReorder(menuReorder);
    } else if (graphSubwindowMode == SELECT_GLYPH) {
        drawMenuGlyph(menuGlyph);
    } else if (graphSubwindowMode == DRAW_GRAPH) {
        drawGraph(graph);
        
        // Print out pass date times
        drawCurPassDateTimesOnGraphScreen();
    } else {
        fprintf(stderr, "ERROR: INVALID graphSubwindowMode: %d\n", graphSubwindowMode);
    }


    glEnd();

   glutSwapBuffers();
}

void graphSubwindowInit(Pass *passIn) {
    graphScreen = createScreen();

    pass = passIn;
        
    graph = createGraph(pass, graphScreen);
    menuShown = createMenuShown(pass, graphScreen);
    menuReorder = createMenuReorder(pass, graphScreen);
    menuGlyph = createMenuGlyph(pass, graphScreen);
}

void graphMouseCallback(int button, int state, int x, int y) {
    // rotate camera
    GLint axis = 3;
    if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN) axis = 0;
    if (button == GLUT_MIDDLE_BUTTON && state == GLUT_DOWN) axis = 1;
    if (button == GLUT_RIGHT_BUTTON && state == GLUT_DOWN) axis = 2;
    if (axis < 3) {  // button ups won't change axis value from 3
        theta[axis] += thetaIncr;
        if (theta[axis] > 360.0) theta[axis] -= 360.0;
        glutPostRedisplay();
    }
}


#define KEY_ENTER       13

void graphKeyCallback(unsigned char key, int x, int y) {
       // move viewer with x, y, and z keys
       // capital moves in + direction, lower-case - direction
       //printf("Current Position: %f, %f, %f\n", modelTrans[0], modelTrans[1], modelTrans[2]);
       //printf("Current Traslation: %f, %f, %f\n", theta[0], theta[1], theta[2]);
       
    if (key == KEY_SHOWN_LOWER || key == KEY_SHOWN_UPPER) {
        // Go to display mode
        if (graphSubwindowMode == DRAW_GRAPH) {
            graphSubwindowMode = SET_VISIBLE_FIELDS;
        } else if (graphSubwindowMode == SET_VISIBLE_FIELDS) {
            graphSubwindowMode = DRAW_GRAPH;
        }

        return;
    }
    

    if (key == KEY_REORDER_LOWER || key == KEY_REORDER_UPPER) {
        // Go to reorder mode
        if (graphSubwindowMode == DRAW_GRAPH) {
            graphSubwindowMode = REORDER_FIELDS;
            resetMenuReorder(menuReorder);
        } else if (graphSubwindowMode == REORDER_FIELDS) {
            graphSubwindowMode = DRAW_GRAPH;
        }
        return;
    }
    
    if (key == KEY_GLYPH_SEL_LOWER || key == KEY_GLYPH_SEL_UPPER) {
        // Go to glyph select mode
        if (graphSubwindowMode == DRAW_GRAPH) {
            graphSubwindowMode = SELECT_GLYPH;
            resetMenuGlyph(menuGlyph);
        } else if (graphSubwindowMode == SELECT_GLYPH) {
            graphSubwindowMode = DRAW_GRAPH;
        }
        return;
    }
    
    if (graphSubwindowMode == SET_VISIBLE_FIELDS) {
        if (key == KEY_ENTER) {
            selectCurrentMenuShownItem(menuShown);
        } else if (key == KEY_CLEAR_DISPLAYED_LOWER || key == KEY_CLEAR_DISPLAYED_UPPER) {
            unshowAllFields(pass);
        } else if (key == KEY_ALL_DISPLAYED_LOWER || key == KEY_ALL_DISPLAYED_UPPER) {
            showAllFields(pass);
        }
                
        
    } else if (graphSubwindowMode == REORDER_FIELDS) {
        if (key == KEY_ENTER) {
            selectCurrentMenuReorderItem(menuReorder);
        }
    }
    else if (graphSubwindowMode == SELECT_GLYPH) {
        if (key == KEY_SELECT_COLOR_GLYPH) {
            selectCurrentAsColorGlyph(menuGlyph);
        } else if (key == KEY_SELECT_SIZE_GLYPH) {
            selectCurrentAsSizeGlyph(menuGlyph);
        }
    }
     else if (graphSubwindowMode == DRAW_GRAPH) {
        if (key == KEY_TOGGLE_GRAPH_LOWER || key == KEY_TOGGLE_GRAPH_UPPER) {
            if(graph->drawType == DRAW_TYPE_BAR) {
                graph->drawType = DRAW_TYPE_CONNECTED;
            } else if(graph->drawType == DRAW_TYPE_CONNECTED) {
                graph->drawType = DRAW_TYPE_LINE;
            } else if(graph->drawType == DRAW_TYPE_LINE) {
                graph->drawType = DRAW_TYPE_BAR;
            }
            
            // Reset camera coordinates
            resetGraphCameraCoordinates();
        }
        else if (key == KEY_TOGGLE_GRAPH_CAM) {
            if(cameraMode == CAMERA_MODE_GLOBAL) {
                cameraMode = CAMERA_MODE_TRACKING;
            }
            else {
                cameraMode = CAMERA_MODE_GLOBAL;
            }
            resetGraphCameraCoordinates();
        }
        else if (key == KEY_MOVE_X_LEFT) modelTrans[0] -= 1.0;
        else if (key == KEY_MOVE_X_RIGHT) modelTrans[0] += 1.0;
        else if (key == KEY_MOVE_Y_DOWN) modelTrans[1] -= 1.0;
        else if (key == KEY_MOVE_Y_UP) modelTrans[1] += 1.0;
        else if (key == KEY_MOVE_Z_FORWARD) modelTrans[2] -= 1.0;
        else if (key == KEY_MOVE_Z_BACK) modelTrans[2] += 1.0;
        
        
        /*else if (key == 'r') {
           theta[0] = 0.0; theta[1] = 0.0; theta[2] = 0.0;
        }
        else if (key == '-') {
           thetaIncr = -thetaIncr;
        }
        else if (key == '+') {
            if (thetaIncr < 0) thetaIncr = thetaIncr - 1.0;
            else               thetaIncr = thetaIncr + 1.0;
        }*/
        else if (key == KEY_TILT_DOWN) {
            theta[0] -= 2;
        }
        else if (key == KEY_TILT_UP) {
            theta[0] += 2;
        }
    }
    
    

    glutPostRedisplay();
}


void graphKeyCallbackSpecial(int key, int x, int y) {
    if (graphSubwindowMode == SET_VISIBLE_FIELDS) {
        if(key == GLUT_KEY_UP) {
            moveMenuShownCursorUp(menuShown);
        } else if(key == GLUT_KEY_DOWN) {
            moveMenuShownCursorDown(menuShown);
        }
    } else if (graphSubwindowMode == REORDER_FIELDS) {
        if(key == GLUT_KEY_UP) {
            moveMenuReorderCursorUp(menuReorder);
        } else if(key == GLUT_KEY_DOWN) {
            moveMenuReorderCursorDown(menuReorder);
        }
    } else if (graphSubwindowMode == SELECT_GLYPH) {
        if(key == GLUT_KEY_UP) {
            moveMenuGlyphCursorUp(menuGlyph);
        } else if(key == GLUT_KEY_DOWN) {
            moveMenuGlyphCursorDown(menuGlyph);
        }
    }
}


void graphChangeSize(int w1, int h1)
    {
	double ratio;

    graphScreen->width  = w1;
    graphScreen->height = h1;

    // Prevent a divide by zero, when window is too short
    // (you cant make a window of zero width).
    if(h1 == 0)
        h1 = 1;
    
    ratio = 1.0f * w1 / h1;
    // Reset the coordinate system before modifying
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    
    // Set the viewport to be the entire window
    glViewport(0, 0, w1, h1);

    // Set the clipping volume
    gluPerspective(45,ratio,0.1,1000);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}


void graphReshapeCallback(int w, int h) {
    // from Angel, p.562
    glViewport(0,0,w,h);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    if (w < h) {
        glFrustum(-2.0, 2.0, -2.0*(GLfloat) h / (GLfloat) w,
                2.0*(GLfloat) h / (GLfloat) w, 2.0, 20.0);
    }
    else {
        glFrustum(-2.0, 2.0, -2.0*(GLfloat) w / (GLfloat) h,
                2.0*(GLfloat) w / (GLfloat) h, 2.0, 20.0);
    }

    glMatrixMode(GL_MODELVIEW);
}

#define CAMERA_X_START 580

#define CUR_PASS_X_START 880

void drawCurPassDateTimesOnGraphScreen(){
    char tempStr[1000];
    char tempStrDateTime[1000];
    
    glDisable(GL_LIGHTING);
    
    glColor3f(1.0,1.0,1.0);
    
    setOrthographicProjection(graphScreen);
    glPushMatrix();
    glLoadIdentity(); 
    
    renderBitmapString(CAMERA_X_START, 20, (void *)subheaderFont, "Camera Mode:");
    
    // Print camera
    if(cameraMode == CAMERA_MODE_GLOBAL) {
        renderBitmapString(CAMERA_X_START + 125, 20, (void *)normalFont, "Global");
    } else {
        renderBitmapString(CAMERA_X_START + 125, 20, (void *)normalFont, "Tracking");
    }

    printDateTimeToArray(pass->fields[0]->data[curSelectedIndex]->dateTime, tempStrDateTime);

    //Current pass time
    renderBitmapString(CUR_PASS_X_START, 20, (void *)subheaderFont, "Current Pass Time:");
    renderBitmapString(CUR_PASS_X_START+160, 20, (void *)normalFont, tempStrDateTime);
    
    glPopMatrix();
    resetPerspectiveProjection();
    
    glEnable(GL_LIGHTING);
}