#include "MenuGlyph.h"

#include <GL/glut.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "DataField.h"
#include "Globals.h"
#include "OpenGLFunctions.h"
#include "Pass.h"
#include "Screen.h"

#define MENU_ITEM_X_START 20
#define MENU_ITEM_Y_START 75

#define MENU_ITEM_WIDTH_SEPARATION 300
#define MENU_ITEM_HEIGHT_SEPARATION 30
#define MENU_NUM_ITEMS_PER_COLUMN 8

#define START_LABEL_OFFSET 4

static Screen *graphScreen;

static int numInShow = 0;

// Array of indicies into the pass (fields)
static int currentlyDisplayed[1000];

MenuGlyph *createMenuGlyph(Pass *passToSet, Screen *graphScreenToSet)
{
    MenuGlyph *toReturn = (MenuGlyph*) malloc(sizeof(MenuGlyph));
    toReturn->pass = passToSet;
    toReturn->cursorPos = 0;
    
    graphScreen = graphScreenToSet;
    
    resetMenuGlyph(toReturn);
    
    return toReturn;
}

/**
 * This method recreates the currently displayed array.
 */
void resetMenuGlyph(MenuGlyph *menuGlyph) {
    int i;
    
    numInShow = 0;
    
    for(i = 0; i < menuGlyph->pass->numFields; i++){
        if(menuGlyph->pass->fields[i]->display == 1){
            currentlyDisplayed[numInShow] = i;
            numInShow++;
        }
    }
}

void drawMenuGlyph(MenuGlyph *menuGlyph) {
    int i;
    int normalFont = (int) GLUT_BITMAP_8_BY_13;
    int headerFont = (int) GLUT_BITMAP_TIMES_ROMAN_24;
    int subheaderFont = (int) GLUT_BITMAP_HELVETICA_12;
    
    char tempLabel[1000];
    tempLabel[0] = ' ';
    tempLabel[1] = ' ';
    tempLabel[2] = ' ';
    tempLabel[3] = ' ';

    setOrthographicProjection(graphScreen);
    glPushMatrix();
    glLoadIdentity();
    
    glColor3f(1.0,1.0,1.0);
    renderBitmapString(MENU_ITEM_X_START,25,(void *)headerFont,"Select which fields should be displayed on orbit");
    renderBitmapString(MENU_ITEM_X_START,40,(void *)subheaderFont,"(Press '1' to select Color and '2' to select Size)");
    
    for(i = 0; i < numInShow; i++) {
        
        if(i == menuGlyph->cursorPos){
            glColor3f(0.1,1.0,0.1);
        } else {
            glColor3f(1.0,1.0,1.0);            
        }
        
        if(currentlyDisplayed[i] == indexOfColorGlyph){
            tempLabel[1] = 'C';
        } else if(currentlyDisplayed[i] == indexOfSizeGlyph){
            tempLabel[1] = 'S';
        } else {
            tempLabel[1] = ' ';
        }
        
        strcpy(tempLabel+START_LABEL_OFFSET, menuGlyph->pass->fields[currentlyDisplayed[i]]->name);
        
        renderBitmapString(MENU_ITEM_X_START + (i/MENU_NUM_ITEMS_PER_COLUMN)*MENU_ITEM_WIDTH_SEPARATION,MENU_ITEM_Y_START + (i%MENU_NUM_ITEMS_PER_COLUMN) * MENU_ITEM_HEIGHT_SEPARATION,(void *)normalFont,tempLabel);
    }
    glPopMatrix();
    resetPerspectiveProjection();

}

void moveMenuGlyphCursorDown(MenuGlyph *menuGlyph){
    menuGlyph->cursorPos++;
    if(menuGlyph->cursorPos >= numInShow){
        menuGlyph->cursorPos = 0;
    }
}

void moveMenuGlyphCursorUp(MenuGlyph *menuGlyph){
    menuGlyph->cursorPos--;
    if(menuGlyph->cursorPos < 0){
        menuGlyph->cursorPos = (numInShow - 1);
    }
}

void selectCurrentAsColorGlyph(MenuGlyph *menuGlyph){
    indexOfColorGlyph = currentlyDisplayed[menuGlyph->cursorPos];
    
    if(indexOfColorGlyph == indexOfSizeGlyph) {
        indexOfSizeGlyph = -1;
    }
}

void selectCurrentAsSizeGlyph(MenuGlyph *menuGlyph){
    indexOfSizeGlyph = currentlyDisplayed[menuGlyph->cursorPos];
    
    if(indexOfColorGlyph == indexOfSizeGlyph) {
        indexOfColorGlyph = -1;
    }
}
