#include "MenuReorder.h"

#include <GL/glut.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "DataField.h"
#include "OpenGLFunctions.h"
#include "Pass.h"
#include "Screen.h"

#define MENU_ITEM_X_START 20
#define MENU_ITEM_Y_START 60

#define MENU_ITEM_WIDTH_SEPARATION 300
#define MENU_ITEM_HEIGHT_SEPARATION 30
#define MENU_NUM_ITEMS_PER_COLUMN 8

#define START_LABEL_OFFSET 4

static Screen *graphScreen;

static int numInShow = 0;

// Array of indicies into the pass (fields)
static int currentlyDisplayed[1000];

static int indexToSwitch = -1;

MenuReorder *createMenuReorder(Pass *passToSet, Screen *graphScreenToSet)
{
    MenuReorder *toReturn = (MenuReorder*) malloc(sizeof(MenuReorder));
    toReturn->pass = passToSet;
    toReturn->cursorPos = 0;
    
    graphScreen = graphScreenToSet;
    
    resetMenuReorder(toReturn);
    
    return toReturn;
}

/**
 * This method recreates the currently displayed array.
 */
void resetMenuReorder(MenuReorder *menuReorder) {
    int i;
    
    numInShow = 0;
    
    for(i = 0; i < menuReorder->pass->numFields; i++){
        if(menuReorder->pass->fields[i]->display == 1){
            currentlyDisplayed[numInShow] = i;
            numInShow++;
        }
    }

    indexToSwitch = -1;
}

void drawMenuReorder(MenuReorder *menuReorder) {
    int i;
    int normalFont = (int) GLUT_BITMAP_8_BY_13;
    int headerFont = (int) GLUT_BITMAP_TIMES_ROMAN_24;
    
    char tempLabel[1000];
    tempLabel[0] = ' ';
    tempLabel[1] = ' ';
    tempLabel[2] = ' ';
    tempLabel[3] = ' ';

    setOrthographicProjection(graphScreen);
    glPushMatrix();
    glLoadIdentity();
    
    glColor3f(1.0,1.0,1.0);
    renderBitmapString(MENU_ITEM_X_START,25,(void *)headerFont,"Select fields to be reordered");
    
    for(i = 0; i < numInShow; i++) {
        
        if(i == menuReorder->cursorPos){
            glColor3f(0.1,1.0,0.1);
        } else {
            glColor3f(1.0,1.0,1.0);            
        }
        
        if(i == indexToSwitch){
            tempLabel[1] = '*';
        } else {
            tempLabel[1] = ' ';
        }
        
        strcpy(tempLabel+START_LABEL_OFFSET, menuReorder->pass->fields[currentlyDisplayed[i]]->name);
        
        renderBitmapString(MENU_ITEM_X_START + (i/MENU_NUM_ITEMS_PER_COLUMN)*MENU_ITEM_WIDTH_SEPARATION,MENU_ITEM_Y_START + (i%MENU_NUM_ITEMS_PER_COLUMN) * MENU_ITEM_HEIGHT_SEPARATION,(void *)normalFont,tempLabel);
    }
    glPopMatrix();
    resetPerspectiveProjection();

}

void moveMenuReorderCursorDown(MenuReorder *menuReorder){
    menuReorder->cursorPos++;
    if(menuReorder->cursorPos >= numInShow){
        menuReorder->cursorPos = 0;
    }
}

void moveMenuReorderCursorUp(MenuReorder *menuReorder){
    menuReorder->cursorPos--;
    if(menuReorder->cursorPos < 0){
        menuReorder->cursorPos = (numInShow - 1);
    }
}

void selectCurrentMenuReorderItem(MenuReorder *menuReorder){
    if(indexToSwitch == -1) {
        // set index
        indexToSwitch = menuReorder->cursorPos;
    } else {
        // Do the swap
        DataField *tempField = menuReorder->pass->fields[currentlyDisplayed[indexToSwitch]];
        menuReorder->pass->fields[currentlyDisplayed[indexToSwitch]] = menuReorder->pass->fields[currentlyDisplayed[menuReorder->cursorPos]];
        menuReorder->pass->fields[currentlyDisplayed[menuReorder->cursorPos]] = tempField;
        
        indexToSwitch = -1;
    }
}
