#include "MenuShown.h"

#include <GL/glut.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "OpenGLFunctions.h"
#include "Pass.h"
#include "Screen.h"

#define MENU_ITEM_X_START 20
#define MENU_ITEM_Y_START 60

#define MENU_ITEM_WIDTH_SEPARATION 300
#define MENU_ITEM_HEIGHT_SEPARATION 30
#define MENU_NUM_ITEMS_PER_COLUMN 8

#define START_LABEL_OFFSET 4

Screen *graphScreen;

MenuShown *createMenuShown(Pass *passToSet, Screen *graphScreenToSet)
{
    MenuShown *toReturn = (MenuShown*) malloc(sizeof(MenuShown));
    toReturn->pass = passToSet;
    toReturn->cursorPos = 0;
    
    graphScreen = graphScreenToSet;
    
    return toReturn;
}


void drawMenuShown(MenuShown *menuShown) {
    int i;
    int normalFont = (int) GLUT_BITMAP_8_BY_13;
    int headerFont = (int) GLUT_BITMAP_TIMES_ROMAN_24;
    
    char tempLabel[1000];
    tempLabel[0] = '[';
    tempLabel[1] = ' ';
    tempLabel[2] = ']';
    tempLabel[3] = ' ';

    setOrthographicProjection(graphScreen);
    glPushMatrix();
    glLoadIdentity();
    
    glColor3f(1.0,1.0,1.0);
    renderBitmapString(MENU_ITEM_X_START,25,(void *)headerFont,"Select which fields should be displayed");
    
    for(i = 0; i < menuShown->pass->numFields; i++) {
        
        if(i == menuShown->cursorPos){
            glColor3f(0.1,1.0,0.1);
        } else {
            glColor3f(1.0,1.0,1.0);            
        }
        
        strcpy(tempLabel+START_LABEL_OFFSET, menuShown->pass->fields[i]->name);
        if(menuShown->pass->fields[i]->display == 1){
            tempLabel[1] = 'X';
        } else {
            tempLabel[1] = ' ';
        }
        
        renderBitmapString(MENU_ITEM_X_START + (i/MENU_NUM_ITEMS_PER_COLUMN)*MENU_ITEM_WIDTH_SEPARATION,MENU_ITEM_Y_START + (i%MENU_NUM_ITEMS_PER_COLUMN) * MENU_ITEM_HEIGHT_SEPARATION,(void *)normalFont,tempLabel);
    }
    glPopMatrix();
    resetPerspectiveProjection();

}

void moveMenuShownCursorDown(MenuShown *menuShown){
    menuShown->cursorPos++;
    if(menuShown->cursorPos >= menuShown->pass->numFields){
        menuShown->cursorPos = 0;
    }
}

void moveMenuShownCursorUp(MenuShown *menuShown){
    menuShown->cursorPos--;
    if(menuShown->cursorPos < 0){
        menuShown->cursorPos = (menuShown->pass->numFields - 1);
    }
}

void selectCurrentMenuShownItem(MenuShown *menuShown){
    if(menuShown->pass->fields[menuShown->cursorPos]->display == 0) {
        menuShown->pass->fields[menuShown->cursorPos]->display = 1;
    } else {
        menuShown->pass->fields[menuShown->cursorPos]->display = 0;
    }
}
