#include "Orbit.h"

#include <GL/glut.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>

#include "CubeSat.h"
#include "DataField.h"
#include "DateTime.h"
#include "Globals.h"
#include "Pass.h"
#include "PassIter.h"
#include "Utils.h"

PassIter *passIter;
OrbitDrawingMode orbitDrawingMode = ORBIT_DRAW_BAR;

Orbit *createOrbit(Pass *passToSet, float inc, float alt, float per) {
    Orbit *toReturn = (Orbit*) malloc(sizeof(Orbit));
    
    toReturn->pass = passToSet;
    
    toReturn->inclination = inc;
    toReturn->altitude = alt;
    toReturn->period = per;    
        
    // Create the pass iter if not done already
    if(passIter == NULL) {
        passIter = createPassIter(passToSet);
    }

    return toReturn;
}

void drawOrbit(Orbit *orbit, float x, float y, float z){
    float angle = 0;
    float angleInterval = PIE2 * (1 / (ORBIT_PERIOD / orbit->pass->interval));
    
   
    float minColorValue = 0.0, maxColorValue = 0.0;
    float colorK = 0.0;
    float colorToUseBlue = 0.0;

	float minSizeValue = 0.0, maxSizeValue = 0.0;
    float sizeK = 0.0;
    float sizeToUse = 0.0;
    float rotAngle = 0.0;

	int curDataIndex;

	int advResult = 0;

    
    // **@** IS THIS CORRECT?
    float timeThatHasPast = timeToSeconds(orbit->pass->startOfPass) - timeToSeconds(orbit->pass->fields[0]->data[curSelectedIndex]->dateTime);
    //(1.0/(24.0*60.0*60.0) is the rate the earth turns
    //printf("1 over: %f\n", timeToSeconds(orbit->pass->startOfPass));
    //printf("time past: %f\n", timeToSeconds(orbit->pass->fields[0]->data[curSelectedIndex]->dateTime));
    //printf("Rotation: %f\n", 360.0 * (timeThatHasPast) * (1.0/(ORBIT_PERIOD)));
    
        
    
    glPushMatrix();
    
    rotAngle = 360.0 * (timeThatHasPast) * (1.0/(ORBIT_PERIOD));
    lastOrbitRotAngle = rotAngle;
    glRotatef(rotAngle, 0.0, 1.0, 0.0);
    
    // COLOR
 
    
    if (indexOfColorGlyph != -1) {
            //printf("Have color field\n");
            minColorValue = getDataFieldMin(orbit->pass->fields[indexOfColorGlyph])->value;
            maxColorValue = getDataFieldMax(orbit->pass->fields[indexOfColorGlyph])->value;
            colorK = (maxColorValue - minColorValue);
    }
    
    // SIZE
    

    
    if (indexOfSizeGlyph != -1) {
            //printf("Have color field\n");
            minSizeValue = getDataFieldMin(orbit->pass->fields[indexOfSizeGlyph])->value;
            maxSizeValue = getDataFieldMax(orbit->pass->fields[indexOfSizeGlyph])->value;
            sizeK = (maxSizeValue - minSizeValue);
    }
    
    
    // Reset the passIter to the current selected index
    resetPassIterFromIndex(passIter,curSelectedIndex);

    
        //glTranslatef(x,y,z);
        //glRotatef(20, 1.0, 0.0, 0.0);
        //glRotatef(50, 0.0, 1.0, 0.0);
        
       
        //printf("Drawing Orbit: %f", angleInterval);
        
        ////
        // Draw Orbit
        ////
       
        
        
        if(orbitDrawingMode != ORBIT_DRAW_SPHERE){
            glDisable(GL_LIGHTING);
        }
        
        
        for(angle = 0; angle < PIE2 && advResult >= 0; angle += angleInterval) {

            glPushMatrix();
                glTranslatef(cos(angle)*(orbit->altitude*1.0),0,sin(angle)*(orbit->altitude*1.0));

                
                curDataIndex = samplePassIter(passIter);
                
                //printf("curSelectedIndex=%d,  curDataIndex=%d\n", curSelectedIndex, curDataIndex);
                
                if(curDataIndex < 0){
                    // Make sure to pop matrix
                    advResult = advancePassIter(passIter);
                    
                    glPopMatrix();
                    continue;
                }
                
                else if (curSelectedIndex == curDataIndex) {
                    //printf("Drawing CubeSat\n");
                    //printf("CurAngle: %f %f\n", cos(angle)*(orbit->altitude*1.0), sin(angle)*(orbit->altitude*1.0));
                    if(orbitDrawingMode == ORBIT_DRAW_SPHERE){
                        // Move it out a bit
                        drawCubeSat(0,-50,0);
                          
                    }
                    else {
                        drawCubeSat(0,0,0);  
                    }
                    
                    
                    lastOrbitXPos = cos(angle)*(orbit->altitude*1.0);
                    lastOrbitYPos = sin(angle)*(orbit->altitude*1.0);
                }
                
                // Set up the color to use
                if (indexOfColorGlyph == -1) {
                    glColor3f(0.5,1.0,0.5);
                }
                else {
                    //printf("min= %f, max= %f, cur=%f",minColorValue,maxColorValue,orbit->pass->fields[indexOfColorGlyph]->data[curDataIndex]->value);
                    colorToUseBlue = (orbit->pass->fields[indexOfColorGlyph]->data[curDataIndex]->value - minColorValue) * colorK;
                    colorToUseBlue = (orbit->pass->fields[indexOfColorGlyph]->data[curDataIndex]->value - minColorValue) / colorK;
                    glColor3f(1-colorToUseBlue,0.0,colorToUseBlue);
                }
                
                
                // Set up the size to use
                if (indexOfSizeGlyph != -1) {
                    
                    //sizeToUse = (orbit->pass->fields[indexOfSizeGlyph]->data[curDataIndex]->value - minSizeValue) / sizeK;
                    float perRange = (orbit->pass->fields[indexOfSizeGlyph]->data[curDataIndex]->value - minSizeValue) / sizeK;
                    sizeToUse =  (perRange * (ORBIT_MAX_SIZE - ORBIT_MIN_SIZE)) + ORBIT_MIN_SIZE;
                    
                    sizeToUse = ORBIT_MAX_SIZE - (sizeToUse - ORBIT_MIN_SIZE);
                    
                    //printf("min= %f, max= %f, cur=%f, sizeToUse=%f\n",minSizeValue,maxSizeValue,orbit->pass->fields[indexOfSizeGlyph]->data[curDataIndex]->value, sizeToUse);
                } else {
                    sizeToUse = ORBIT_DEFAULT_SIZE;
                }
                
                
                if (orbitDrawingMode == ORBIT_DRAW_BAR){
                    glDisable(GL_LIGHTING);
                    // **@** Add base line
                    
                    glBegin(GL_QUADS);
                    // Draw Front
                    glNormal3d(0, 0, 1);
                    glVertex3f(-ORBIT_BAR_WIDTH, 0, -ORBIT_BAR_WIDTH);
                    glVertex3f(ORBIT_BAR_WIDTH, 0, -ORBIT_BAR_WIDTH);
                    glVertex3f(ORBIT_BAR_WIDTH, sizeToUse, -ORBIT_BAR_WIDTH);
                    glVertex3f(-ORBIT_BAR_WIDTH, sizeToUse, -ORBIT_BAR_WIDTH);
            
                    // Draw Left
                    glNormal3d(-1, 0, 0);
                    glVertex3f(-ORBIT_BAR_WIDTH, 0, -ORBIT_BAR_WIDTH);
                    glVertex3f(-ORBIT_BAR_WIDTH, sizeToUse, -ORBIT_BAR_WIDTH);
                    glVertex3f(-ORBIT_BAR_WIDTH, sizeToUse, ORBIT_BAR_WIDTH);
                    glVertex3f(-ORBIT_BAR_WIDTH, 0, ORBIT_BAR_WIDTH);
                    
                    // Draw Back
                    glNormal3d(0, 0, -1);
                    glVertex3f(-ORBIT_BAR_WIDTH, 0, ORBIT_BAR_WIDTH);
                    glVertex3f(-ORBIT_BAR_WIDTH, sizeToUse, ORBIT_BAR_WIDTH);
                    glVertex3f(ORBIT_BAR_WIDTH, sizeToUse, ORBIT_BAR_WIDTH);
                    glVertex3f(ORBIT_BAR_WIDTH, 0, ORBIT_BAR_WIDTH);
                    
                    // Draw Right
                    glNormal3d(1, 0, 0);
                    glVertex3f(ORBIT_BAR_WIDTH, 0, -ORBIT_BAR_WIDTH);
                    glVertex3f(ORBIT_BAR_WIDTH, 0, ORBIT_BAR_WIDTH);
                    glVertex3f(ORBIT_BAR_WIDTH, sizeToUse, ORBIT_BAR_WIDTH);
                    glVertex3f(ORBIT_BAR_WIDTH, sizeToUse, -ORBIT_BAR_WIDTH);
                            
                    // Draw Top
                    glNormal3d(0, 1, 0);
                    glVertex3f(-ORBIT_BAR_WIDTH, sizeToUse, -ORBIT_BAR_WIDTH);
                    glVertex3f(ORBIT_BAR_WIDTH, sizeToUse, -ORBIT_BAR_WIDTH);
                    glVertex3f(ORBIT_BAR_WIDTH, sizeToUse, ORBIT_BAR_WIDTH);
                    glVertex3f(-ORBIT_BAR_WIDTH, sizeToUse, ORBIT_BAR_WIDTH);

                    glEnd();
                    
                    glEnable(GL_LIGHTING);
                    
                } else if (orbitDrawingMode == ORBIT_DRAW_SPHERE){
                    
                    
                    glutSolidSphere(sizeToUse/5.0, 20, 20);
                } else {
                    fprintf(stderr, "Invalid Orbit Draw Mode: %d\n", orbitDrawingMode);
                }
                
            glPopMatrix();
            
            advResult = advancePassIter(passIter);
        }
        
        if(orbitDrawingMode != ORBIT_DRAW_SPHERE){
            glEnable(GL_LIGHTING);
        }
        
    glPopMatrix();
}

void cycleOrbitDrawType() {
    if(orbitDrawingMode == ORBIT_DRAW_BAR) {
        orbitDrawingMode = ORBIT_DRAW_SPHERE;
    } else if(orbitDrawingMode == ORBIT_DRAW_SPHERE) {
        orbitDrawingMode = ORBIT_DRAW_BAR;
    }
}
