#include "Pass.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "DateTime.h"
#include "DataField.h"
#include "SatData.h"

#include <math.h>

#define LINE_BUF_SIZE 100000

Pass *createPass(const char *inputFile)
{
	Pass *toReturn = (Pass*) malloc(sizeof(Pass));
	int strLen;
	char *tempStr;
	int i,j = 0;
	DateTime *curDateTime;
	float dataValue;
	int max;
	
	char lineBuf[LINE_BUF_SIZE];
	FILE *f = fopen(inputFile, "r");

	// Init fields
	toReturn->numFields = 0;
    toReturn->interval = PASS_DATA_INTERVAL;
	
	if(f == NULL){
		printf("Input file '%s' not found\n", inputFile);
		exit(-1);
	}
	
	fgets(lineBuf, LINE_BUF_SIZE, f);
	printf("%s\n", lineBuf);
	
	// Build String Tokenizer
	tempStr = strtok(lineBuf, ",");
	
	
	while(1){
		
		tempStr = strtok(NULL, ",");
		
		
		/* check if there is nothing else to extract */
        if (tempStr == NULL)
        {
        	break;
        }
		
		toReturn->fields[toReturn->numFields] = createDataField();
        dataFieldCopyName(toReturn->fields[toReturn->numFields], tempStr);
        if (isSatFieldName(tempStr)) {
        	toReturn->fields[toReturn->numFields]->display = 1;
        }
        else {
        	toReturn->fields[toReturn->numFields]->display = 0;
        }
		
		//printf("Adding: %s\n",  toReturn->fields[toReturn->numFields]->name);
		
		toReturn->numFields++;
	}
	
	while( fgets(lineBuf, LINE_BUF_SIZE, f) != NULL){
		tempStr = strtok(lineBuf, ",");
		
		// Set up datetime
		
		curDateTime = createDateTime(tempStr);
		//curDateTime.print();
		
		for(i = 0; i < toReturn->numFields; i++){
			tempStr = strtok(NULL, ",");
		
			// check if there is nothing else to extract 
        	if (tempStr == NULL)
        	{
        		break;
        	}
        	
        	dataValue = atof(tempStr);
        	//dataValue = log(dataValue);
        	
        	dataFieldAddData(toReturn->fields[i], curDateTime, dataValue);
        	//exit(1);
		}
	}
	
	// Normalize Data
	for(i = 0 ; i < toReturn->numFields; i++){
		DataField *curField = toReturn->fields[i];
		
		max = 0;
		for(j = 0 ; j < curField->numData; j++){
			if(curField->data[j]->value > max){
				max = curField->data[j]->value;
			}
		}
		
		if(max > 5){
			for(j = 0 ; j < curField->numData; j++){
				curField->data[j]->value  = curField->data[j]->value/max*5;
			}
		}
	}

	fclose(f);
    
    
    toReturn->numDataPoints = toReturn->fields[0]->numData;
    toReturn->startOfPass = toReturn->fields[0]->data[0]->dateTime;
    
    // populate datetime string
    printDateTimeToArray(toReturn->fields[0]->data[0]->dateTime,toReturn->dateTimeStr);
    strLen = strlen(toReturn->dateTimeStr);
    toReturn->dateTimeStr[strLen] = ' ';
    toReturn->dateTimeStr[strLen+1] = '-';
    toReturn->dateTimeStr[strLen+2] = ' ';
    
    printDateTimeToArray(toReturn->fields[0]->data[toReturn->numDataPoints-1]->dateTime,toReturn->dateTimeStr+strLen+3);
    printDateTime(toReturn->fields[0]->data[toReturn->numDataPoints-1]->dateTime);
    printf("%s\n",toReturn->dateTimeStr);
    
	return toReturn;
}

void printPass(Pass *pass)
{
	int i = 0;
	
	for(i = 0; i < pass->numFields; i++){
		printDataField(pass->fields[i]);
	}
}

DateTime *getPassStartTime(Pass *pass){
    return pass->fields[0]->data[0]->dateTime;
}

void swapPassDataFields(Pass *pass, int toSwap1, int toSwap2){
    DataField *temp = pass->fields[toSwap1];
    pass->fields[toSwap1] = pass->fields[toSwap2];
    pass->fields[toSwap2] = temp;
}

/**
 * returns the time for the given index
 */
DateTime *getTimeForDataFieldIndex(Pass *pass, int index) {
    return pass->fields[0]->data[index]->dateTime;
}

int nextDisplayField(Pass *pass, int curDisplayIndex) {
    int i = 0;
    
    // To the end
    for(i = curDisplayIndex+1;i < pass->numFields; i++){
        if(pass->fields[i]->display == 1){
            return i;
        }
    }
    
    // From the beginning
    for(i = 0; i < curDisplayIndex; i++){
        if(pass->fields[i]->display == 1){
            return i;
        }
    }
    
    return -1;
}

int prevDisplayField(Pass *pass, int curDisplayIndex) {
    int i = 0;
    
    // To the beginning
    for(i = curDisplayIndex-1; i >= 0; i--){
        if(pass->fields[i]->display == 1){
            return i;
        }
    }
    
    // From the end
    for(i = pass->numFields-1; i > curDisplayIndex; i--){
        if(pass->fields[i]->display == 1){
            return i;
        }
    }
    
    return -1;
}

void unshowAllFields(Pass *pass) {
    int i = 0;
    
    for(i = 0; i < pass->numFields; i++) {
        pass->fields[i]->display = 0;
    }
}

void showAllFields(Pass *pass) {
    int i = 0;
    
    for(i = 0; i < pass->numFields; i++) {
        pass->fields[i]->display = 1;
    }
}

int numOfDisplayedFields(Pass *pass){
}
