#include "PassIter.h"


#include <stdlib.h>
#include <stdio.h>

#include "Pass.h"
#include "DateTime.h"
#include "DebugLog.h"

#define TOLERANCE_ON_TIME_INTERVAL 15 // in seconds

PassIter *createPassIter(Pass *toSetPass)
{
	PassIter *toReturn = (PassIter*) malloc(sizeof(PassIter));
    toReturn->pass = toSetPass;
	toReturn->curLocation = 0;
    toReturn->curTime = timeToSeconds(getPassStartTime(toSetPass));
	
	return toReturn;
}

void resetPassIter(PassIter *toReset)
{
    toReset->curLocation = 0;
    toReset->curTime = timeToSeconds(toReset->pass->startOfPass);
    toReset->dataIndex = 0;
}

void resetPassIterFromIndex(PassIter *toReset, int startIndex)
{
    toReset->curLocation = startIndex;
    toReset->curTime = timeToSeconds(toReset->pass->fields[0]->data[startIndex]->dateTime);
    toReset->dataIndex = startIndex;
}

void destroyPassIter(PassIter *passIter) {
    free(passIter);
}

void printPassIter(PassIter *passIter)
{
	printf("Cur Data Index: %d\n", passIter->dataIndex);
}

/**
 * @return the current index of the data for the iterator
 */
int samplePassIter(PassIter *passIter) {
    return passIter->dataIndex;
}

/**
 * This function advances the passIter to the next interval in the orbit.
 * 
 * @return  0 If successful
 *         -1 if not
 */
char advancePassIter(PassIter *passIter) {
    Pass *curPass = passIter->pass;
    int nextIndex = passIter->curLocation + 1;
	unsigned int nextDataTime;
    
    printDebug("Starting advancePassIter()");
    
    // Are we at the end?    
    if(curPass->numDataPoints == nextIndex) {
        // At end of pass so cannot advance
        printDebug("Ending advancePassIter() with -1");
        return -1;
    }
    
    
    
    // Advance the time
    passIter->curTime = passIter->curTime + curPass->interval;
       
    nextDataTime = timeToSeconds(getTimeForDataFieldIndex(curPass, nextIndex));
    
    //printf("nextTime:%d  ,  curTime:%d\n",  nextDataTime, passIter->curTime);
    
    if(passIter->curTime+TOLERANCE_ON_TIME_INTERVAL > nextDataTime) {
        passIter->curTime = nextDataTime;
        passIter->curLocation += 1;
        passIter->dataIndex = passIter->curLocation;
    } else {
        passIter->dataIndex = -1;
    }
    
    printDebug("Ending advancePassIter()");
    
    return 0;
}
